classdef Drift_Diff_N
    %DRIFT_DIFF Summary of this class goes here
    %   Detailed explanation goes here
    %8/16/2011,add generation term
    
    properties

        pb1% boundary condition for hole 
        pbn% boundary condition(end) for hole 
        nb1% boundary condition for electron
        nbn% boundary condition(end) for electron
        wb1% boundary condition for potential
        wbn% boundary condition(end) for potential
        fd
        mesh
        Semiconductor_Device

    end
    
    methods
        function self=Drift_Diff_N(pb1,pbn,nb1,nbn,wb1,wbn,fd,mesh,Semiconductor_Device)
            self.pb1=pb1;
            self.pbn=pbn;
            self.nb1=nb1;
            self.nbn=nbn;
            self.wb1=wb1;
            self.wbn=wbn;
            self.fd = fd;
            self.mesh=mesh;
            self.Semiconductor_Device = Semiconductor_Device;

        end
        
        function [Fp,Fn,Fw,J_T]=Cal_Current(self,pm,nm,phim)
            % calculate the function for drift&diffusion equation and
            % possion equation
            % the following are the parameters for the device.
            Nmu=self.Semiconductor_Device.NP.Nmu; % normlized mobolity;
            %D0=self.Semiconductor_Device.NP.D0; % normlized diffusion
            p=[self.pb1;pm;self.pbn];
            n=[self.nb1;nm;self.nbn];
            phi=[self.wb1;phim;self.wbn];
            N1=self.mesh.N1;
            N=self.mesh.N;
            %q = self.Semiconductor_Device.q; 
            %Ebslon = self.Semiconductor_Device.Ebslon;
            tau_n = self.Semiconductor_Device.Lifetime_n;
            ni = self.Semiconductor_Device.Intrinsic_Density;
            Doping = self.Semiconductor_Device.Doping_Profile;
            n1=self.Semiconductor_Device.n1;
            p1=self.Semiconductor_Device.p1;
            % incomplete ionization model
            Doping_ion=zeros(N,1);
            Doping_ion(1:N1)=Doping(1:N1)./(1+4*p(1:N1)./p1(1:N1));
            Doping_ion(N1+1:end)=Doping(N1+1:end)./(1+2*n(N1+1:end)./n1(N1+1:end));
            %T_Vol = self.Semiconductor_Device.T_Vol;
            Electron_Mobility = self.Semiconductor_Device.Electron_Mobility;
            Hole_Mobility = self.Semiconductor_Device.Hole_Mobility;
            %%%%%%%%%%%%
            
            E = -self.fd.diffmp(phi);% electrical field
            Electron_Velocity = E*Electron_Mobility; % Electron Velocity
            Hole_Velocity = E*Hole_Mobility; % Hole Velocity
            %Electron_Duffision = T_Vol*Electron_Mobility; % Electron Diffusion 
            %Hole_Duffision = T_Vol*Hole_Mobility;
            
            Recombination=(n.*p)./(tau_n.*(n+p+2*ni)); % Recombination-ni.*ni
            
            
            %Recombination=p*0;
            % the impact ionization generation
            % G_imp=alph_An*n*abs(vn)+alpha_Ap*p*abs(vp)
            % this vn and vp should be on the main point;
            if self.Semiconductor_Device.Impact_Ion
            Ae = 6.64e6*self.Semiconductor_Device.NP.NX;
            Be = 2.2e6*self.Semiconductor_Device.NP.NX/self.Semiconductor_Device.NP.VT;
            Ah = 0*self.Semiconductor_Device.NP.NX;
            Bh = 2.26e6*self.Semiconductor_Device.NP.NX/self.Semiconductor_Device.NP.VT;
            
            E_main = -self.fd.diff3p(phi); % the electric field at the main point
            Vn_main = E_main*Electron_Mobility; % Electron Velocity at the main point
            Vp_main = E_main*Hole_Mobility; % Hole Velocity at the main point
            alpha_An = Ae.*exp(-Be./abs(E_main));
            alpha_Ap = Ah.*exp(-Bh./abs(E_main));
            G_imp = +alpha_Ap.*p(2:end-1).*abs(Vp_main)+alpha_An.*n(2:end-1).*abs(Vn_main);
            else
                G_imp=0;
            end
            J_Hole = Hole_Velocity.*self.fd.averagemp(p)...% Hole current,there should be a q times, but it cancelled in the Fp.
                - Hole_Mobility.*self.fd.diffmp(p);
%             J_Hole(self.mesh.N2)=0;
            
            J_Elec = Electron_Velocity.*self.fd.averagemp(n)...% electron current
                + Electron_Mobility.*self.fd.diffmp(n); % electron current
            J_T=J_Hole+J_Elec;
            Fp = -self.fd.diffmidp(J_Hole)-Recombination(2:end-1)+G_imp; % drift % diffusion current equation for hole
            Fn = self.fd.diffmidp(J_Elec)-Recombination(2:end-1)+G_imp;% drift % diffusion current equation for electron
            dx=self.fd.mesh.dx;
            dx1=self.fd.mesh.dx1;
            Fw = (Doping_ion(2:end-1)+p(2:end-1)-n(2:end-1))... % ni(2:end-1)is norm
                +(1./((dx(1:end-1)).*(dx1(1:end)))).*phi(1:end-2)...
                -((1./dx(1:end-1)+1./dx(2:end))./dx1(1:end)).*phi(2:end-1)...
                +(1./((dx(2:end)).*(dx1(1:end)))).*phi(3:end);% possion equation
            
%             r1=1/(h(Mi-1)*h1(Ni));
%                 r2=-(1/h(Mi-1)+1/h(Mi))/h1(Ni);
%                 r3=1/(h(Mi)*h1(Ni));
%                 
%                 f3=-q*(Tol(Ni)+p(Ni)-n(Ni))/s-r1*w(Ni-1)-r2*w(Ni)-r3*w(Ni+1)
            %boundary condition
            
%             Fw(1) = Fw(1)-phi(1)/(dx(1)*dx1(1));
%             Fw(end) = Fw(end)-phi(end)/(dx(end)*dx1(end-1));
        end
        function [JacobM]=ConsJacobian(self,pm,nm,phim)
            Nmu=self.Semiconductor_Device.NP.Nmu; % normlized mobolity;
            p=[self.pb1;pm;self.pbn];
            n=[self.nb1;nm;self.nbn];
            phi=[self.wb1;phim;self.wbn];

            %T_Vol=self.Semiconductor_Device.T_Vol;
%             q = self.Semiconductor_Device.q;
%             Ebslon = self.Semiconductor_Device.Ebslon;
            tau_n = self.Semiconductor_Device.Lifetime_n;
            tau_p = self.Semiconductor_Device.Lifetime_p;
            ni = self.Semiconductor_Device.Intrinsic_Density;
            Doping = self.Semiconductor_Device.Doping_Profile;
            n1=self.Semiconductor_Device.n1;
            p1=self.Semiconductor_Device.p1;
            N=self.mesh.N;
            N1=self.mesh.N1;
            % incomplete ionization model
            Doping_ion=zeros(N,1);
            Doping_ion(1:N1)=Doping(1:N1)./(1+4*p(1:N1)./p1(1:N1));
            Doping_ion(N1+1:end)=Doping(N1+1:end)./(1+2*n(N1+1:end)./n1(N1+1:end));
            % Doing_ion  derive to the p and n
            Doping_ion_p=zeros(N,1);
            Doping_ion_n=zeros(N,1);
            Doping_ion_p(1:N1)=-Doping_ion(1:N1)./(p1(1:N1)/4+p(1:N1));
            Doping_ion_n(N1+1:end)=-Doping_ion(N1+1:end)./(n1(N1+1:end)/2+n(N1+1:end));
            
            Electron_Mobility = self.Semiconductor_Device.Electron_Mobility;
            Hole_Mobility = self.Semiconductor_Device.Hole_Mobility;
            
            E = -self.fd.diffmp(phi);
            %dx1(2:N-1)=0.5*(dx(1:N-2)+dx(2:N-1)); % The spacing between the middle nodes
            
            
            Recombination=(n.*p)./(tau_n.*(n+p+2*ni));
            % Recombination rate R=(p*n-ni^2)/(lifetime*(p+n+2*ni))
            %recombination derivative to the carriers.
%             DUDp=p*0;
            DUDp=(n-tau_n.*Recombination)./...
                            (tau_n.*(p+ni)...
                            +tau_p.*(n+ni));
            % first derivative  Recombination(i) to Hole_Density(i)
            DUDn=(p-tau_p.*Recombination)./...
                            (tau_n.*(p+ni)+tau_p.*(n+ni));
%             DUDn=n*0;
%           % the impact ionization generation
            % G_imp=alph_An*n*abs(vn)+alpha_Ap*p*abs(vp)
            % this vn and vp should be on the main point;
            if  self.Semiconductor_Device.Impact_Ion
            Ae = 6.64e6*self.Semiconductor_Device.NP.NX;
            Be = 2.2e6*self.Semiconductor_Device.NP.NX/self.Semiconductor_Device.NP.VT;
            Ah = 0*self.Semiconductor_Device.NP.NX;
            Bh = 2.26e6*self.Semiconductor_Device.NP.NX/self.Semiconductor_Device.NP.VT;
            
            E_main = -self.fd.diff3p(phi); % the electric field at the main point
            Vn_main = E_main*Electron_Mobility; % Electron Velocity at the main point
            Vp_main = E_main*Hole_Mobility; % Hole Velocity at the main point
            alpha_An = Ae.*exp(-Be./abs(E_main));
            alpha_Ap = Ah.*exp(-Bh./abs(E_main));
            %G_imp = alpha_An*n(2:end-1).*abs(Vn_main)+alpha_Ap*p(2:end-1).*abs(Vp_main); 
            % the derive of G_imp to n,p and w
            DG_impDn = alpha_An.*abs(Vn_main);
            DG_impDp = alpha_Ap.*abs(Vp_main);
            DG_impDE = alpha_An.*n(2:end-1).*abs(Vn_main).*Be*1./(E_main.^2).*sign(E_main)...
                     + alpha_An.*n(2:end-1).*Electron_Mobility.*sign(E_main)...
                     + alpha_Ap.*p(2:end-1).*abs(Vp_main).*Bh*1./E_main.^2.*sign(E_main)...
                     + alpha_Ap.*p(2:end-1).*Hole_Mobility.*sign(E_main);
            DG_impDwm = DG_impDE./(self.mesh.dx(1:end-1)+self.mesh.dx(2:end));% DG_imp to w_i-1  
            DG_impDwp = -DG_impDwm;
            else
                DG_impDn=0;
                DG_impDp=0;
                DG_impDwm=zeros(N-2,1);
                DG_impDwp=zeros(N-2,1);
                  
            end
            %DG_impDE=G_imp.*Be*1./E_main^2.*sign(E_main)+alpha_An.*n(2:end-1).*Electron_Mobility.*sign(E_main);
            %CreateJacobM()
            
            
            Electron_Velocity=E*Electron_Mobility; % Electron Velocity
            Hole_Velocity=E*Hole_Mobility; % Hole Velocity
            
            
            %Electron_Duffision=207.2; % electron duffision coeificient, einstein relation
            %Hole_Duffision=7.77;% hole duffision coeificient
            % J_Hole and J_ Electron first derivative to p ,n and phi
            DJpDp = Hole_Velocity/2+Hole_Mobility.*self.fd.differ;% first derivative hole current J_Hole(i) to Hole_Density(i)
            DJpDp2 = Hole_Velocity/2-Hole_Mobility.*self.fd.differ;% first derivative J_Hole(i) to Hole_Density(i+1)
            
            DJpDw = Hole_Mobility.*self.fd.averagemp(p).*self.fd.differ; % first derivative J_Hole(i) to Potential(i)
            DJpDw2 = -DJpDw; %  % first derivative J_Hole(i) to Potential(i+1)
            
%             DJpDp(self.mesh.N2)=0;
%             DJpDp2(self.mesh.N2)=0;
%             DJpDw(self.mesh.N2)=0;
%             DJpDw2(self.mesh.N2)=0;
%             
            DJnDn = Electron_Velocity/2-Electron_Mobility.*self.fd.differ; % % first derivative electron current J_Elec(i) to Elec_Density(i)
            DJnDn2 = Electron_Velocity/2+Electron_Mobility.*self.fd.differ; % first derivative current J_Elec(i) to Elec_Density(i+1)
            DJnDw = Electron_Mobility.*(self.fd.averagemp(n)).*self.fd.differ;  % first derivative  J_Elec(i) to potential(i)
            DJnDw2 = -DJnDw; % first derivative  J_Elec(i) to potential(i+1)
            %calculate the current of hole and electrons
            %J_Hole = self.q*Hole_Velocity.*self.fd.averagemp(p) ...
            %             - self.q*Hole_Duffision.*self.fd.diffmp(p);
            
            % Hole current, the first part is the drift current
            % Jdrift=q*p*V(E), here we use the average of the two nodes to
            % get the middle node of the hole density, the same for the
            % electron current.The second part is the diffusion current.
            % Jdiff=-q*Dp*dp/dx.
            %J_Elec = self.q*Electron_Velocity.*self.fd.averagemp(n) ...
            %+ self.q*Electron_Duffision.*self.fd.diffmp(n); % electron current
            % Electron current, the first part is the drift current
            % Jdrift=q*n*V(E), here we use the average of the two nodes to
            % get the middle node of the electron density. The second part is the diffusion current.
            % Jdiff=q*Dn*dn/dx.
            %constuct the Jacob matrix to solve the equation for the hole
                                 %continuity and electron continuity(Fp, Fn) and possion
                                 %equation(Fw)
                                 %Fp(2:N-1)
                                 %Fn(2:N-1)
                                 %Fw(2:N-1)
                                 %so the Jacob matrix***DFp/Dp,DFp/Dn,DFp/Dw************
                                 %**********************DFn/Dp,DFn/Dn,DFn/Dw************
                                 %**********************DFw/Dp,DFw/Dn,DFw/Dw************
            dx1=self.mesh.dx1; 
            Num=length(dx1)+2;
            % Fp derivative to p , n and phi
            a=[DJpDp(2:end-1)./(dx1(2:end));0];
            e=(-DJpDp(2:end)+DJpDp2(1:end-1))./(dx1(1:end))-DUDp(2:end-1)+DG_impDp;%(2:N-1)
            b=[0;-DJpDp2(2:end-1)./(dx1(1:end-1))];
            M1=spdiags([a e b],-1:1,Num-2,Num-2); % Fp first derivative to p(hole denstiy)
            M2=spdiags(-DUDn(2:end-1)+DG_impDn,0,Num-2,Num-2); % Fp first derivative to n (electron denstiy)
            a=[DJpDw(2:end-1)./(dx1(2:end))+DG_impDwm(2:end);0];
            e=(-DJpDw(2:end)+DJpDw2(1:end-1))./(dx1(1:end));
            b=[0;-DJpDw2(2:end-1)./(dx1(1:end-1))+DG_impDwp(1:end-1)];
            M3=spdiags([a e b],-1:1,Num-2,Num-2);% Fp first derivative to potential
            MFp=[M1,M2,M3]; % Fp first derivative to p ,n and potential
            
            % Fn derive to p n w
            M1=spdiags(-DUDp(2:end-1)+DG_impDp,0,Num-2,Num-2); % Fn first derivative to p(hole denstiy)
            a=[-DJnDn(2:end-1)./(dx1(2:end));0];
            e=(DJnDn(2:end)-DJnDn2(1:end-1))./(dx1(1:end))-DUDn(2:end-1)+DG_impDn;
            b=[0;DJnDn2(2:end-1)./(dx1(1:end-1))];
            M2=spdiags([a e b],-1:1,Num-2,Num-2); % Fn first derivative to n(electron denstiy)
            a=[-DJnDw(2:end-1)./(dx1(2:end))+DG_impDwm(2:end);0]; % diag-1,row :i, col; i-1
            e=(DJnDw(2:end)-DJnDw2(1:end-1))./(dx1(1:end)); % diag
            b=[0;DJnDw2(2:end-1)./(dx1(1:end-1))+DG_impDwp(1:end-1)];
            M3=spdiags([a e b],-1:1,Num-2,Num-2); % Fn first derivative to potential
            MFn=[M1,M2,M3];
            
            %                 DEDx = self.fd.diffmidp(E);%Electrical Field First derivative to x
            %                 DFpDp = Hole_Mobility/2*diag(DEDx)+Hole_Mobility/2*diag(DEDx)*self.fd.CreateDiffAve+diag(self.fd.averagemp(Hole_Velocity))*self.fd.CreateFMOM...
            %                       -diag(self.fd.averagemp(Hole_Duffision))*self.fd.CreateSOM-diag(DUDp(2:end-1)); % Fp first derivative to p
            %                 DFpDphi = Hole_Mobility*(diag(p(2:end-1))*self.fd.CreateSOM+diag(self.fd.diff3p(p))*self.fd.CreateFMOM);% Fp first derivative to phi
            %                 DFpDn = -diag(DUDn(2:end-1)); % Fp first derivative to n
            %
            %                 DFnDp = -diag(DUDp(2:end-1));% Fn first derivative to p
            %                 DFnDn = Electron_Mobility/2*diag(DEDx)+Electron_Mobility/2*diag(DEDx)*self.fd.CreateDiffAve + diag(self.fd.averagemp(Electron_Velocity))*self.fd.CreateFMOM...
            %                       + diag(self.fd.averagemp(Electron_Duffision))*self.fd.CreateSOM-diag(DUDn(2:end-1));% Fn first derivative to n
            %                 DFnDphi = -Electron_Mobility*(diag(n(2:end-1))*self.fd.CreateSOM - diag(self.fd.diff3p(n))*self.fd.CreateFMOM);
            % Fn first derivative to phi
            DFphiDp = self.fd.CreateZOM+spdiags(Doping_ion_p(2:end-1),0,Num-2,Num-2);% Fphi first derivative to p
                    %+spdiags(Doping_ion_p(2:end-1),0,Num-2,Num-2);
            DFphiDn = -self.fd.CreateZOM+spdiags(Doping_ion_n(2:end-1),0,Num-2,Num-2); % Fphi first derivative to n
            DFphiDphi= self.fd.CreateSOM; % Fp first derivative to phi
            
            
            JacobM = [MFp;MFn;DFphiDp,DFphiDn,DFphiDphi];
        end
        function [y,M]=Newton_FD(self,x)
%             pb1 = self.pb1;
%             pbn = self.pbn;
%             nb1 = self.nb1;
%             nbn = self.nbn;
%             wb1=self.wb1;
%             wbn=self.wbn;
            N=length(self.mesh.dx)+1;
            pm = x(1:N-2);
            nm = x(N-1:2*N-4);
            wm = x(2*N-3:3*N-6);

            [Fp,Fn,Fw]=self.Cal_Current(pm,nm,wm);
            y = [Fp;Fn;Fw];
            M = self.ConsJacobian(pm,nm,wm);
        end
        function [y,M]=Newton_FD_Time(self,x)
%             pb1 = self.pb1;
%             pbn = self.pbn;
%             nb1 = self.nb1;
%             nbn = self.nbn;
%             wb1=self.wb1;
%             wbn=self.wbn;
            Deltat=self.deltat;
            N=length(self.mesh.dx)+1;
            pm = x(1:N-2);
            nm = x(N-1:2*N-4);
            wm = x(2*N-3:3*N-6);
            x0=x(3*N-5:end);
            pm0 = x0(1:N-2);
            nm0 = x0(N-1:2*N-4);
            %wm = x(2*N-3:3*N-6);
            Ft=([pm-pm0;nm-nm0;wm-wm])/Deltat;
            
            [Fp,Fn,Fw]=self.Cal_Current(pm,nm,wm);
            y = [Fp;Fn;Fw]-Ft;
            
            M = self.ConsJacobian(pm,nm,wm)+self.Mt;
        end
        
        
    end
    
end

