classdef Drift_Diff_N_C_GN < handle
    %DRIFT_DIFF Summary of this class goes here
    %   Detailed explanation goes here
    % non linearlity analysis
    
    properties

        pb1% boundary condition for hole 
        pbn% boundary condition(end) for hole 
        nb1% boundary condition for electron
        nbn% boundary condition(end) for electron
        wb1% boundary condition for potential
        wbn% boundary condition(end) for potential
        fd
        mesh
        SS % the semiconductor _device parameter
        NP % normlized x parameter
        Nmu% normlized mobility
        BF% boundary conditionflag 1 for true using boundary, 0 for no
        Mt % matrix for solving the transient problem should be update in the program
        x0
        time % the light parameter including time, intenstiy, and frequency
        GL
        FL
        Steptime % steptime in transit simulation
        % the coeificient for the diffusion current
        alphap
        alphan
    end
    
    methods
        function self=Drift_Diff_N_C_GN(pb1,pbn,nb1,nbn,wb1,wbn,fd,mesh,Semiconductor_Device,NP,BF,alphap,alphan)
            self.pb1=pb1;
            self.pbn=pbn;
            self.nb1=nb1;
            self.nbn=nbn;
            self.wb1=wb1;
            self.wbn=wbn;
            self.fd = fd;
            self.mesh=mesh;
            self.SS = Semiconductor_Device;
            self.NP=NP;
            self.BF=BF;
            
            self.alphap=alphap;
            self.alphan=alphan;
%             self.Mt=Mt;
%             self.x0=x0;
%             self.time=realtime;
%             self.GL=GL;
%             self.FL=FL;
%             self.Steptime=Steptime;
        end
        
        
        function [Fp,Fn,Fw,J_Hole,J_Elec,J_Hole_Dr,J_Hole_Du,Jn_drift,Jn_diff,Doping_ion,NAp,NDp]=Cal_Current(self,pm,nm,phim,time,GL,FL)
            % calculate the function for drift&diffusion equation and
            % possion equation
            % the following are the parameters for the device.
            N2=self.mesh.N2;
            N1=self.mesh.N1;
            N=self.mesh.N;
            BF=self.BF;
            VT=self.SS.T_Vol;
            p=[self.pb1;pm;self.pbn];
            n=[self.nb1;nm;self.nbn];
            phi=[self.wb1;phim;self.wbn];
            Vps=self.SS.Mater.Vps;
            Vns=self.SS.Mater.Vns;
            beta=self.SS.Mater.beta;
            Ep=self.SS.Ep;
            gama=self.SS.gama;
            R_Ebslon = self.SS.Mater.R_Ebslon;
            Phi = self.SS.Mater.Phi;
            Eg = self.SS.Mater.Eg;
            %q = self.SS.q; 
            %Ebslon = self.SS.Ebslon;
            tau_n = self.SS.Lifetime_n;
            ni = self.SS.Intrinsic_Density;
            Doping = self.SS.Doping_Profile;
            n1=self.SS.n1; % used in the incomplete ionization model
            p1=self.SS.p1; % used in the incomplete ionization model
            % incomplete ionization model
            Doping_ion=zeros(N,1);
            NAp=zeros(N,1);
            NDp=zeros(N,1);
            Doping_ion(1:N1)=Doping(1:N1)./(1+4*p(1:N1)./p1(1:N1));
            Doping_ion(N1+1:end)=Doping(N1+1:end)./(1+2*n(N1+1:end)./n1(N1+1:end));
            NAp(1:N1)=abs(Doping_ion(1:N1));
            NDp(N1+1:end)=abs(Doping_ion(N1+1:end));
            %T_Vol = self.SS.T_Vol;
%             Electron_Mobility = self.SS.Electron_Mobility;
%             Hole_Mobility = self.SS.Hole_Mobility;
            %%%%%%%%%%%%
            %%
            Electron_Mobility=self.SS.Mater.un;
            Hole_Mobility=self.SS.Mater.up;
%             Hole_Mobility=Hole_Mobility./(1+(abs(Doping)/1e18*self.NP.Ni).^0.45);
%             % mobility doping dependent
%             Electron_Mobility=Electron_Mobility./(1+(abs(Doping)/1e17*self.NP.Ni).^0.5);
            %%
            
            E = -self.fd.diffmp(phi);% electrical field
            %Electron_Velocity = E*Electron_Mobility; % Electron Velocity
            %Hole_Velocity = E*Hole_Mobility; % Hole Velocity
            % use the saturated velocity
            %             Electron_Velocity=E.*(Electron_Mobility+Vns*beta*abs(E))./(1+beta*E.^2);
            %             Hole_Velocity=E*Hole_Mobility*Vps./(Vps^4+Hole_Mobility^4*(E.^4)).^(1/4);
            %first do not use normolized parameter to calculate the
            %velocity and diffusion coeificient
            %Eu=E*VT/self.NX;
            %Nmu=self.SS.NP.Nmu;
            VT=self.SS.NP.VT;
            NX=self.NP.NX;
            %Eu=E*VT/NX;
            %Electron_Mobility1=Electron_Mobility*Nmu;
            %gama=4;
            Electron_Velocity = E.*((Electron_Mobility(2:end)+Vns(2:end).*beta(2:end).*abs(E))./(1+beta(2:end).*(E.^2)));
%             Electron_Velocity = E.*((Electron_Mobility(2:end)+Vns*beta2*abs(E))./(1+beta2*(E.^2)));
%             Electron_Velocity(1:self.mesh.Node(9)-1) = Electron_Velocity1(1:self.mesh.Node(9)-1);
            
            Hole_Velocity = E.*Vps(2:end).*Hole_Mobility(2:end)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^(1./gama)); % Hole Velocity
            Electron_Duffision = self.alphan*Electron_Mobility(2:end)./((1-2*(abs(E)/Ep).^2+4/3*(abs(E)/Ep).^3).^(1/4));
            Hole_Duffision = self.alphap*Hole_Mobility(2:end).*Vps(2:end)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^(1./gama));
            %Electron_Duffision = T_Vol*Electron_Mobility; % Electron Diffusion 
            %Hole_Duffision = T_Vol*Hole_Mobility;
            
            Recombination=1*(n.*p-ni.^2)./(tau_n.*(n+p+2*ni)); % Recombination-ni.*ni
%             Cn=1e-31*ones(N,1);
%             %Cn(self.mesh.Node(9)+1:end)=2e-28;
%             Cp=Cn;
%             Br=1e-11*ones(N,1);
%             Br(self.mesh.Node(9)+1:end)=1e-10;
%             Br=0;
            Cp=self.SS.Mater.Cp;
            Cn=self.SS.Mater.Cn;
            Br=self.SS.Mater.Br;
            R_Auger=(Cn.*n+Cp.*p).*(n.*p-ni.^2); % Auger recombination
            R_Rad = Br.*(n.*p-ni.^2);% Radiative recombination
             % the impact ionization generation
             % G_imp=alph_An*n*abs(vn)+alpha_Ap*p*abs(vp)
             % this vn and vp should be on the main point;
             E_main = -self.fd.diff3p(phi);
             if self.SS.Impact_Ion
                 Ae = self.SS.Ae*self.SS.NP.NX; % 6.64e7
                 Be = self.SS.Be*self.SS.NP.NX/self.SS.NP.VT; % 2e6;
                 Ah = self.SS.Ah*self.SS.NP.NX; % 9.34e7
                 Bh = self.SS.Bh*self.SS.NP.NX/self.SS.NP.VT; % 2.26e6
                 
                 % the electric field at the main point
                 Vn_main = E_main.*((Electron_Mobility(2:end-1)+Vns(2:end-1).*beta(2:end-1).*abs(E_main))./(1+beta(2:end-1).*(E_main.^2))); % Electron Velocity at the main point
                 Vp_main = E_main.*(Vps(2:end-1).*Hole_Mobility(2:end-1)./((Vps(2:end-1).^gama+Hole_Mobility(2:end-1).^gama.*(abs(E_main).^gama)).^(1./gama))); % Hole Velocity at the main point
                 Dn_main= Hole_Mobility(2:end-1).*Vps(2:end-1)./((Vps(2:end-1).^gama+Hole_Mobility(2:end-1).^gama.*(abs(E_main).^gama)).^(1./gama));
                 In = self.SS.In;
                 Ip = self.SS.Ip;
                 alpha_An = Ae.*exp(-(Be./abs(E_main)).^In);
                 alpha_Ap = Ah.*exp(-(Bh./abs(E_main)).^Ip);
                 
                 G_imp = +alpha_Ap.*p(2:end-1).*abs(Vp_main)+alpha_An.*abs(n(2:end-1).*Vn_main+0*Dn_main.*self.fd.diff3p(n));
                 G_imp(self.mesh.Node(6):end)=0;
             else
                 G_imp=0;
             end
             
            alpha=self.SS.alpha0;% unit,cm-1
            G0=GL*(0.4*sin(2*pi*FL*time)+1);
            % pulse test
%             if time<1.1e-15 
%                 G0=(1e-4);
%             else
%                 G0=0;
%             end
%            G0=1e-3;
            E_main_all=[E_main(1);E_main;E_main(end)];
            if self.SS.Franz==1 % franz-keldshy effoct
                alpha0=Absorp_FKO(abs(E_main_all*(VT/NX))*100,1.216e15,Eg,alpha); % 0.73 is Eg 1.216e15
                alpha0(1:self.mesh.Node(5))=alpha;
		alpha0(self.mesh.Node(6)+1:end)=alpha;
            elseif self.SS.Franz==2
                alpha0=Franz_compbell(abs(E_main_all*(VT/NX))*100,1.216e15,Eg,Phi);
		alpha0(1:self.mesh.Node(5))=alpha;
		alpha0(self.mesh.Node(6)+1:end)=alpha;
            else
                alpha0=alpha;
            end
            G0=G0.*alpha0/14000;
            Generation=G0.*exp(-alpha0.*(0.99e-4-(self.NP.NX*self.mesh.Lx)));
            Generation(self.mesh.N_absorp+1:N)=0;
            Generation(self.mesh.Node(1)+1:self.mesh.Node(2))=0;
            Generation(1:N1)=self.SS.alpha_p*Generation(1:N1);
            Generation(self.mesh.Node(5)+1:self.mesh.Node(6))=0.4*Generation(self.mesh.Node(5)+1:self.mesh.Node(6));
            Generation(self.mesh.Node(6)+1:end)=0.5*self.SS.alpha_n*Generation(self.mesh.Node(6)+1:end);
            %Recombination=p*0;
            J_Hole_Dr=Hole_Velocity.*self.fd.averagemp(p);% drift current
            J_Hole_Du=- Hole_Duffision.*self.fd.diffmp(p);% diffusion current
%             if BF
% 
% 
%                     J_Hole_Du(self.mesh.N2)=0;
% 
%             end
            J_Hole = J_Hole_Dr+J_Hole_Du;% Hole current,there should be a q times, but it cancelled in the Fp.
            if BF % thermionic emission model at the heterojunction
                E1=E(self.mesh.Node(9));
                E2=E1;
                x1=self.mesh.dx(self.mesh.Node(9))/2;
                x2=x1;
                delEv=0.5;
                temp_up=p(self.mesh.Node(9))*exp(E1*x1)*exp(delEv)-p(self.mesh.Node(9)+1)*exp(-E2*x2);
                temp_down=Hole_Velocity(self.mesh.Node(9))/Vps*exp(delEv)+(1-exp(-E2*x2))-(1-exp(-E1*x1))*exp(delEv);
                J_Hole(self.mesh.Node(9))=Hole_Velocity(self.mesh.Node(9))*temp_up/temp_down;
            end
%              J_Hole(self.mesh.N2)=0;  
            Jn_drift=Electron_Velocity.*self.fd.averagemp(n);
            Jn_diff=Electron_Duffision.*self.fd.diffmp(n);
            J_Elec = Jn_drift...% electron drift current
                   + Jn_diff; % electron diffusion current
            J_T=J_Hole+J_Elec;
            J_Tdiff=self.fd.diffmidp(J_T);
            Fp = -self.fd.diffmidp(J_Hole)-Recombination(2:end-1)-R_Auger(2:end-1)-R_Rad(2:end-1)+Generation(2:end-1)+G_imp; % drift % diffusion current equation for hole
            Fn = self.fd.diffmidp(J_Elec)-Recombination(2:end-1)-R_Auger(2:end-1)-R_Rad(2:end-1)+Generation(2:end-1)+G_imp;% drift % diffusion current equation for electron
            dx=self.fd.mesh.dx;
            dx1=self.fd.mesh.dx1;
            Fw = (Doping_ion(2:end-1)+p(2:end-1)-n(2:end-1))... % ni(2:end-1)is norm
                +(1./R_Ebslon(2:end-1)).*((1./((dx(1:end-1)).*(dx1(1:end)))).*phi(1:end-2)...
                -((1./dx(1:end-1)+1./dx(2:end))./dx1(1:end)).*phi(2:end-1)...
                +(1./((dx(2:end)).*(dx1(1:end)))).*phi(3:end))...
                -(self.fd.diff3p(phi).*self.fd.diff3p(R_Ebslon)./R_Ebslon(2:end-1).^2);;% possion equation
            
%             r1=1/(h(Mi-1)*h1(Ni));
%                 r2=-(1/h(Mi-1)+1/h(Mi))/h1(Ni);
%                 r3=1/(h(Mi)*h1(Ni));
%                 
%                 f3=-q*(Tol(Ni)+p(Ni)-n(Ni))/s-r1*w(Ni-1)-r2*w(Ni)-r3*w(Ni+1)
            %boundary condition
            
%             Fw(1) = Fw(1)-phi(1)/(dx(1)*dx1(1));
%             Fw(end) = Fw(end)-phi(end)/(dx(end)*dx1(end-1));
        end
        function [JacobM,MNADt]=ConsJacobian(self,pm,nm,phim)
            p=[self.pb1;pm;self.pbn];
            n=[self.nb1;nm;self.nbn];
            phi=[self.wb1;phim;self.wbn];
            BF=self.BF;
            %T_Vol=self.SS.T_Vol;
%             q = self.SS.q;
%             Ebslon = self.SS.Ebslon;
            Vps=self.SS.Mater.Vps;
            Vns=self.SS.Mater.Vns;
            beta=self.SS.Mater.beta;
            R_Ebslon = self.SS.Mater.R_Ebslon;
            
            gama=self.SS.gama;
            Ep=self.SS.Ep;
            tau_n = self.SS.Lifetime_n;
            tau_p = self.SS.Lifetime_p;
            ni = self.SS.Intrinsic_Density;
            Doping = self.SS.Doping_Profile;
            N=self.mesh.N;
            N1=self.mesh.N1;
            % incomplete ionization model
            n1=self.SS.n1; % used in the incomplete ionization model
            p1=self.SS.p1; % used in the incomplete ionization model
            Doping_ion=zeros(N,1);
            Doping_ion(1:N1)=Doping(1:N1)./(1+4*p(1:N1)./p1(1:N1));
            Doping_ion(N1+1:end)=Doping(N1+1:end)./(1+2*n(N1+1:end)./n1(N1+1:end));
            % Doing_ion  derive to the p and n
            Doping_ion_p=zeros(N,1);
            Doping_ion_n=zeros(N,1);
            Doping_ion_p(1:N1)=-Doping_ion(1:N1)./(p1(1:N1)/4+p(1:N1));
            Doping_ion_n(N1+1:end)=-Doping_ion(N1+1:end)./(n1(N1+1:end)/2+n(N1+1:end));
            tempa=[-Doping_ion_p(2:N-1);Doping_ion_n(2:N-1);zeros(N-2,1)];
            MNADt=spdiags(tempa,0,3*(N-2),3*(N-2));
%             Electron_Mobility = self.SS.Electron_Mobility;
%             Hole_Mobility = self.SS.Hole_Mobility;
            %%
            Electron_Mobility=self.SS.Mater.un;
            Hole_Mobility=self.SS.Mater.up;
%             Hole_Mobility=Hole_Mobility./(1+(abs(Doping)/1e18*self.NP.Ni).^0.45);
%             Electron_Mobility=Electron_Mobility./(1+(abs(Doping)/1e17*self.NP.Ni).^0.5);
            E = -self.fd.diffmp(phi);
            %dx1(2:N-1)=0.5*(dx(1:N-2)+dx(2:N-1)); % The spacing between the middle nodes
            
            
            Recombination=1*(n.*p-ni.^2)./(tau_n.*(n+p+2*ni));
            % Recombination rate R=(p*n-ni^2)/(lifetime*(p+n+2*ni))
            %recombination derivative to the carriers.
%             DUDp=p*0;
            DUDp=1*(n-tau_n.*Recombination)./...
                            (tau_n.*(p+ni)...
                            +tau_p.*(n+ni));
            % first derivative  Recombination(i) to Hole_Density(i)
            DUDn=1*(p-tau_p.*Recombination)./...
                            (tau_n.*(p+ni)+tau_p.*(n+ni));
%             Cn=1e-31*ones(N,1);
%             %Cn(self.mesh.Node(9)+1:end)=2e-28;
%             Cp=Cn;
%             Br=1e-11*ones(N,1);
%             Br(self.mesh.Node(9)+1:end)=1e-10;
%             Br=0;
            Cp=self.SS.Mater.Cp;
            Cn=self.SS.Mater.Cn;
            Br=self.SS.Mater.Br;
            R_Auger=(Cn.*n+Cp.*p).*(n.*p-ni.^2); % Auger recombination
            R_Rad = Br.*(n.*p-ni.^2);
            
            DR_AugerDp=Cp.*(n.*p-ni.^2)+(Cn.*n+Cp.*p).*n;
            DR_AugerDn=Cn.*(n.*p-ni.^2)+(Cn.*n+Cp.*p).*p;
            DR_RadDp=Br.*n;
            DR_RadDn=Br.*p;
            DUDp=DUDp+DR_AugerDp+DR_RadDp;
            DUDn=DUDn+DR_AugerDn+DR_RadDn;
%             DUDn=n*0;
%             
            %CreateJacobM()
            % the impact ionization generation
            % G_imp=alph_An*n*abs(vn)+alpha_Ap*p*abs(vp)
            % this vn and vp should be on the main point;
            if self.SS.Impact_Ion
                Ae = self.SS.Ae*self.SS.NP.NX; % 6.64e7
                Be = self.SS.Be*self.SS.NP.NX/self.SS.NP.VT; % 2e6;
                Ah = self.SS.Ah*self.SS.NP.NX; % 9.34e7
                Bh = self.SS.Bh*self.SS.NP.NX/self.SS.NP.VT; % 2.26e6
                
                E_main = -self.fd.diff3p(phi); % the electric field at the main point
                Vn_main = E_main.*((Electron_Mobility(2:end-1)+Vns(2:end-1).*beta(2:end-1).*abs(E_main))./(1+beta(2:end-1).*(E_main.^2))); % Electron Velocity at the main point
                Vp_main = E_main.*(Vps(2:end-1).*Hole_Mobility(2:end-1)./((Vps(2:end-1).^gama+Hole_Mobility(2:end-1).^gama.*(abs(E_main).^gama)).^(1./gama))); % Hole Velocity at the main point
                In = self.SS.In;
                Ip = self.SS.Ip;
                alpha_An = Ae.*exp(-(Be./abs(E_main)).^In);
                alpha_Ap = Ah.*exp(-(Bh./abs(E_main)).^Ip);
                G_imp = +alpha_Ap.*p(2:end-1).*abs(Vp_main)+alpha_An.*n(2:end-1).*abs(Vn_main);
                % the derive of G_imp to n,p and w
                DVnmainDE = (1-beta(2:end-1).*(E_main.^2)).*(Electron_Mobility(2:end-1)+Vns(2:end-1).*beta(2:end-1).*abs(E_main))./((1+beta(2:end-1).*(E_main.^2)).^2)+...
                    E_main.*(Vns(2:end-1).*beta(2:end-1).*sign(E_main))./(1+beta(2:end-1).*(E_main.^2));
                DVpmainDE = Hole_Mobility(2:end-1).*Vps(2:end-1)./((Vps(2:end-1).^gama+Hole_Mobility(2:end-1).^gama.*(abs(E_main).^gama)).^(1./gama))...
                    -Hole_Mobility(2:end-1).^(gama+1).*Vps(2:end-1).*(abs(E_main).^gama)./((Vps(2:end-1).^gama+Hole_Mobility(2:end-1).^gama.*(abs(E_main).^gama)).^((gama+1)./gama));
                DG_impDn = alpha_An.*abs(Vn_main);
                DG_impDp = alpha_Ap.*abs(Vp_main);
                DG_impDE = alpha_An.*n(2:end-1).*abs(Vn_main).*Be*In./(abs(E_main).^(1+In)).*sign(E_main)...
                    + alpha_An.*n(2:end-1).*DVnmainDE.*sign(E_main)...
                    + alpha_Ap.*p(2:end-1).*abs(Vp_main).*Bh*Ip./abs(E_main).^(1+Ip).*sign(E_main)...
                    + alpha_Ap.*p(2:end-1).*DVpmainDE.*sign(E_main);
                DG_impDwm = DG_impDE./(self.mesh.dx(1:end-1)+self.mesh.dx(2:end));% DG_imp to w_i-1
                DG_impDwp = -DG_impDwm;
            else
                DG_impDn=0;
                DG_impDp=0;
                DG_impDwm=zeros(N-2,1);
                DG_impDwp=zeros(N-2,1);
                  
            end           
            
%%   the capped current          
%             Electron_Velocity=E.*(Electron_Mobility+Vns*beta*abs(E))./(1+beta*E.^2); % electron velocity
%             Hole_Velocity=E*Hole_Mobility*Vps./((Vps^4+Hole_Mobility^4*(E.^4)).^(1/4));% hole velocity
            VT=self.SS.NP.VT;
            Nmu=self.SS.NP.Nmu;
            NX=self.NP.NX;
            EN=VT/NX;
            %Eu=E*EN;
            %Electron_Mobility1=Electron_Mobility*Nmu;
            Electron_Velocity = E.*((Electron_Mobility(2:end)+Vns(2:end).*beta(2:end).*abs(E))./(1+beta(2:end).*(E.^2)));
%             Electron_Velocity = E.*((Electron_Mobility(2:end)+Vns*beta2*abs(E))./(1+beta2*(E.^2)));
%             Electron_Velocity(1:self.mesh.Node(9)-1) = Electron_Velocity1(1:self.mesh.Node(9)-1);
            %gama=4;
            Hole_Velocity = E.*Vps(2:end).*Hole_Mobility(2:end)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^(1./gama)); % Hole Velocity
            Electron_Duffision = self.alphan*Electron_Mobility(2:end)./((1-2*(abs(E)/Ep).^2+4/3*(abs(E)/Ep).^3).^(1/4));
            Hole_Duffision = self.alphap*Hole_Mobility(2:end).*Vps(2:end)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^(1./gama));
            DVnDE = (1-beta(2:end).*(E.^2)).*(Electron_Mobility(2:end)+Vns(2:end).*beta(2:end).*abs(E))./((1+beta(2:end).*(E.^2)).^2)+...
                E.*(Vns(2:end).*beta(2:end).*sign(E))./(1+beta(2:end).*(E.^2));
            
%             DVnDE = (1-beta2*(E.^2)).*(Electron_Mobility(2:end)+Vns*beta2.*abs(E))./((1+beta2*(E.^2)).^2)+...
%                 E.*(Vns*beta2*sign(E))./(1+beta2*(E.^2));
%             DVnDE(1:self.mesh.Node(9)-1)=DVnDE1(1:self.mesh.Node(9)-1);
%             DVnDE=(1-beta*(E.^2)*EN^2).*(Electron_Mobility1+Vns*beta.*abs(E*EN))./(Nmu*(1+beta*(E.^2)*EN^2).^2)+...
%                 E.*(Vns*beta*sign(E*EN))./((1+beta*(E.^2)*EN^2)*Nmu);
            DVpDE = Hole_Mobility(2:end).*Vps(2:end)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^(1./gama))...
                -Hole_Mobility(2:end).^(gama+1).*Vps(2:end).*(abs(E).^gama)./((Vps(2:end).^gama+Hole_Mobility(2:end).^gama.*(abs(E).^gama)).^((gama+1)./gama));
            
            DDnDE = self.alphan*Electron_Mobility(2:end).*((E)/Ep^2-sign(E).*(E.^2)/Ep^3)./((1-2*(E/Ep).^2+4/3*(abs(E)/Ep).^3).^(5/4));
            DDpDE = -self.alphap*Hole_Mobility(2:end).^(gama+1).*Vps(2:end).*(abs(E).^(gama-1)).*sign(E)./((Vps(2:end).^(gama)+Hole_Mobility(2:end).^(gama).*(abs(E).^(gama))).^((gama+1)./gama));
%%            
            %Electron_Duffision=207.2; % electron duffision coeificient, einstein relation
            %Hole_Duffision=7.77;% hole duffision coeificient
            % J_Hole and J_ Electron first derivative to p ,n and phi
            DJpDp = Hole_Velocity/2+Hole_Duffision.*self.fd.differ;% first derivative hole current J_Hole(i) to Hole_Density(i)
            DJpDp2 = Hole_Velocity/2-Hole_Duffision.*self.fd.differ;% first derivative J_Hole(i) to Hole_Density(i+1)
            
            DJpDw = DVpDE.*self.fd.averagemp(p).*self.fd.differ-DDpDE.*self.fd.diffmp(p).*self.fd.differ; % first derivative J_Hole(i) to Potential(i)
            DJpDw2 = -DJpDw; %  % first derivative J_Hole(i) to Potential(i+1)
            % boundary condition
%             if BF
% 
%                 
%                     dp=self.fd.diffmp(p);
%                     sfd=self.fd.differ;
%                     temp1=DVpDE.*self.fd.averagemp(p).*self.fd.differ;
%                     DJpDp(self.mesh.N2)=Hole_Velocity(self.mesh.N2)/2;
%                     DJpDp2(self.mesh.N2)=Hole_Velocity(self.mesh.N2)/2;
%                     DJpDw(self.mesh.N2)=temp1(self.mesh.N2);
%                     DJpDw2(self.mesh.N2)=-DJpDw(self.mesh.N2);
% 
%             end
            Nhe=self.mesh.Node(9);
            if BF % this is the thermionic model in the heterojunction
                E1=E(Nhe);
                E2=E1;
                x1=self.mesh.dx(Nhe)/2;
                x2=x1;
                delEv=0.5;
                temp_up=p(Nhe)*exp(E1*x1)*exp(delEv)-p(Nhe+1)*exp(-E2*x2);
                temp_down=Hole_Velocity(Nhe)/Vps*exp(delEv)+(1-exp(-E2*x2))-(1-exp(-E1*x1))*exp(delEv);
                DJpDp(Nhe)=Hole_Velocity(Nhe)*exp(E1*x1)*exp(delEv)/temp_down;
                DJpDp2(Nhe)=-Hole_Velocity(Nhe)*exp(-E2*x2)*exp(delEv)/temp_down;
                
                DJpDw(Nhe)=1/self.mesh.dx(Nhe)*(temp_up/temp_down*DVpDE(Nhe)+(Hole_Velocity(Nhe)*(p(Nhe)*exp(E1*x1)*exp(delEv)*x1+p(Nhe+1)*exp(-E2*x2)*x2)/temp_down)...
                          -Hole_Velocity(Nhe)*temp_up*(exp(-E2*x2)*x2+exp(E1*x1)*exp(delEv)*x1+exp(delEv)/Vps*DVpDE(Nhe))/(temp_down)^2);
                DJpDw2(Nhe)=-DJpDw(Nhe);     
                
            end
%             DJpDp(self.mesh.N2)=0;
%             DJpDp2(self.mesh.N2)=0;
%             DJpDw(self.mesh.N2)=0;
%             DJpDw2(self.mesh.N2)=0;
            DJnDn = Electron_Velocity/2-Electron_Duffision.*self.fd.differ; % % first derivative electron current J_Elec(i) to Elec_Density(i)
            DJnDn2 = Electron_Velocity/2+Electron_Duffision.*self.fd.differ; % first derivative current J_Elec(i) to Elec_Density(i+1)
            DJnDw = DVnDE.*(self.fd.averagemp(n)).*self.fd.differ+DDnDE.*self.fd.diffmp(n).*self.fd.differ;  % first derivative  J_Elec(i) to potential(i)
            DJnDw2 = -DJnDw; % first derivative  J_Elec(i) to potential(i+1)
            %calculate the current of hole and electrons
            %J_Hole = self.q*Hole_Velocity.*self.fd.averagemp(p) ...
            %             - self.q*Hole_Duffision.*self.fd.diffmp(p);
            
            % Hole current, the first part is the drift current
            % Jdrift=q*p*V(E), here we use the average of the two nodes to
            % get the middle node of the hole density, the same for the
            % electron current.The second part is the diffusion current.
            % Jdiff=-q*Dp*dp/dx.
            %J_Elec = self.q*Electron_Velocity.*self.fd.averagemp(n) ...
            %+ self.q*Electron_Duffision.*self.fd.diffmp(n); % electron current
            % Electron current, the first part is the drift current
            % Jdrift=q*n*V(E), here we use the average of the two nodes to
            % get the middle node of the electron density. The second part is the diffusion current.
            % Jdiff=q*Dn*dn/dx.
            %constuct the Jacob matrix to solve the equation for the hole
                                 %continuity and electron continuity(Fp, Fn) and possion
                                 %equation(Fw)
                                 %Fp(2:N-1)
                                 %Fn(2:N-1)
                                 %Fw(2:N-1)
                                 %so the Jacob matrix***DFp/Dp,DFp/Dn,DFp/Dw************
                                 %**********************DFn/Dp,DFn/Dn,DFn/Dw************
                                 %**********************DFw/Dp,DFw/Dn,DFw/Dw************
            dx1=self.mesh.dx1; 
            Num=length(dx1)+2;
            % Fp derivative to p , n and phi
            a=[DJpDp(2:end-1)./(dx1(2:end));0];
            e=(-DJpDp(2:end)+DJpDp2(1:end-1))./(dx1(1:end))-DUDp(2:end-1)+DG_impDp;%(2:N-1)
            b=[0;-DJpDp2(2:end-1)./(dx1(1:end-1))];
            M1=spdiags([a e b],-1:1,Num-2,Num-2); % Fp first derivative to p(hole denstiy)
            M2=spdiags(-DUDn(2:end-1)+DG_impDn,0,Num-2,Num-2); % Fp first derivative to n (electron denstiy)
            a=[DJpDw(2:end-1)./(dx1(2:end))+DG_impDwm(2:end);0];
            e=(-DJpDw(2:end)+DJpDw2(1:end-1))./(dx1(1:end));
            b=[0;-DJpDw2(2:end-1)./(dx1(1:end-1))+DG_impDwp(1:end-1)];
            M3=spdiags([a e b],-1:1,Num-2,Num-2);% Fp first derivative to potential
            MFp=[M1,M2,M3]; % Fp first derivative to p ,n and potential
            
            % Fn derive to p n w
            M1=spdiags(-DUDp(2:end-1)+DG_impDp,0,Num-2,Num-2); % Fn first derivative to p(hole denstiy)
            a=[-DJnDn(2:end-1)./(dx1(2:end));0];
            e=(DJnDn(2:end)-DJnDn2(1:end-1))./(dx1(1:end))-DUDn(2:end-1)+DG_impDn;
            b=[0;DJnDn2(2:end-1)./(dx1(1:end-1))];
            M2=spdiags([a e b],-1:1,Num-2,Num-2); % Fn first derivative to n(electron denstiy)
            a=[-DJnDw(2:end-1)./(dx1(2:end))+DG_impDwm(2:end);0]; % diag-1,row :i, col; i-1
            e=(DJnDw(2:end)-DJnDw2(1:end-1))./(dx1(1:end)); % diag
            b=[0;DJnDw2(2:end-1)./(dx1(1:end-1))+DG_impDwp(1:end-1)];
            M3=spdiags([a e b],-1:1,Num-2,Num-2); % Fn first derivative to potential
            MFn=[M1,M2,M3];
            
            %                 DEDx = self.fd.diffmidp(E);%Electrical Field First derivative to x
            %                 DFpDp = Hole_Mobility/2*diag(DEDx)+Hole_Mobility/2*diag(DEDx)*self.fd.CreateDiffAve+diag(self.fd.averagemp(Hole_Velocity))*self.fd.CreateFMOM...
            %                       -diag(self.fd.averagemp(Hole_Duffision))*self.fd.CreateSOM-diag(DUDp(2:end-1)); % Fp first derivative to p
            %                 DFpDphi = Hole_Mobility*(diag(p(2:end-1))*self.fd.CreateSOM+diag(self.fd.diff3p(p))*self.fd.CreateFMOM);% Fp first derivative to phi
            %                 DFpDn = -diag(DUDn(2:end-1)); % Fp first derivative to n
            %
            %                 DFnDp = -diag(DUDp(2:end-1));% Fn first derivative to p
            %                 DFnDn = Electron_Mobility/2*diag(DEDx)+Electron_Mobility/2*diag(DEDx)*self.fd.CreateDiffAve + diag(self.fd.averagemp(Electron_Velocity))*self.fd.CreateFMOM...
            %                       + diag(self.fd.averagemp(Electron_Duffision))*self.fd.CreateSOM-diag(DUDn(2:end-1));% Fn first derivative to n
            %                 DFnDphi = -Electron_Mobility*(diag(n(2:end-1))*self.fd.CreateSOM - diag(self.fd.diff3p(n))*self.fd.CreateFMOM);
            % Fn first derivative to phi
            DFphiDp = self.fd.CreateZOM+spdiags(Doping_ion_p(2:end-1),0,Num-2,Num-2);% Fphi first derivative to p
                    %+spdiags(Doping_ion_p(2:end-1),0,Num-2,Num-2);
            DFphiDn = -self.fd.CreateZOM+spdiags(Doping_ion_n(2:end-1),0,Num-2,Num-2); % Fphi first derivative to n
            DFphiDphi= self.fd.CreateSOM*spdiags(1./R_Ebslon(2:end-1),0,Num-2,Num-2)...
                -spdiags(a,0,Num-2,Num-2).*self.fd.CreateFDOM; % Fp first derivative to phi
            
            
            JacobM = [MFp;MFn;DFphiDp,DFphiDn,DFphiDphi];
        end
        function [y,M]=Newton_FD(self,x)
%             pb1 = self.pb1;
%             pbn = self.pbn;
%             nb1 = self.nb1;
%             nbn = self.nbn;
%             wb1=self.wb1;
%             wbn=self.wbn;
            N=length(self.mesh.dx)+1;
            pm = x(1:N-2);
            nm = x(N-1:2*N-4);
            wm = x(2*N-3:3*N-6);

            [Fp,Fn,Fw]=self.Cal_Current(pm,nm,wm);
            y = [Fp;Fn;Fw];
            M = self.ConsJacobian(pm,nm,wm);
        end
        function self=updatetransit_matrix(self,Mt,Ft,time,GL,FL)
            self.Mt=Mt;
            self.Ft=Ft;
            self.time=time;
            self.GL=GL;
            self.FL=FL;
        end
        function [y,M]=Newton_FD_t(self,x)
%             pb1 = self.pb1;
%             pbn = self.pbn;
%             nb1 = self.nb1;
%             nbn = self.nbn;
%             wb1=self.wb1;
%             wbn=self.wbn;
            N=length(self.mesh.dx)+1;
            pm = x(1:N-2);
            nm = x(N-1:2*N-4);
            wm = x(2*N-3:3*N-6);
            pm0 = self.x0(1:N-2);
            nm0 = self.x0(N-1:2*N-4);
            Ft=([pm-pm0;nm-nm0;wm-wm])/self.Steptime;
            [Fp,Fn,Fw]=self.Cal_Current(pm,nm,wm,self.time,self.GL,self.FL);
            y = [Fp;Fn;Fw]-Ft;
            M = self.ConsJacobian(pm,nm,wm)+self.Mt;
        end
        
        
        
    end
    
end

