% this si the main program to solve photodetector
% add the resistance in the circuit
% add the incomplete ionization model in the simulation
warning off
%clear
%Parameters_NR
% initial condition of the hole and electron density
% ohmic contact p-NA_ion-n=0 and p*n=ni^2;NA_I=NA/(1+gA*p/p1), p1=NV*exp(-(EA-EV)/kT);
% then we calculate the equations to get the p at the boundary
% EA-EV=0.19eV, and EC_ED=0.07eV;
DeltaEA=0.025; % 0.025
DeltaED=0.005; % 0.005
% Impact_Ion=1;% use the impact ionization model
R_load=0;

% Ae=6.64e7;
% Be=2e6;
% Ah=9.34e7;
% Bh=2.26e6;
p1=NV*exp(-DeltaEA/NP.VT);
n1=NC*exp(-DeltaED/NP.VT);
% boundary condition for the carrier density for ohmic contact
pb1=(-p1(1)+sqrt(p1(1)^2+16*p1(1)*abs(Tol(1))))/8;
nb1=ni(1)^2/pb1;
nbn=(-n1(end)+sqrt(n1(end)^2+8*n1(end)*abs(Tol(end))))/4;
pbn=ni(N)^2/nbn;
% NA_I% the ionization acceptor density NA_I=NA/(1+gA*p/p1), p1=NV*exp(-(EA-EV)/kT);
% ND_I% the ionization dopor density ND_I=ND/(1+gD*n/n1), n1=NC*exp(-(EC-ED)/kT);
% NB_I% the ionization dopor density NB_I=NB/(1+gD*n/n1), n1=NC*exp(-(EC-ED)/kT);

% pb1=NA;
% nb1=ni(1)^2/pb1;
% nbn=ND;
% pbn=ni(N)^2/nbn;

p(1:N1,1)=pb1;  % initial condition of the hole density for the first region. It depends on the acceptor doping density.
n(1:N1,1)=ni(1:N1,1).^2./p(1:N1,1); % in the equilibrium condition, p*n=ni^2.

n(N1+1:N2,1)=ni(N1+1:N2);% initial condition of the electron density for the second region. It depends on the donor doping density.
p(N1+1:N2,1)=ni(N1+1:N2,1);% in the equilibrium condition, p*n=ni^2

n(N2+1:N,1)=nbn;% initial condition of the electron density for the third region. It depends on the donor doping density.
p(N2+1:N,1)=ni(N2+1:N,1).^2./n(N2+1:N,1);% in the equilibrium condition, p*n=ni^2
%n(2:N2,1)=tan(1.565695+(1.570796-1.565695).*[0:N2-2]/(N2-2)); % this is tan function to create a smooth electron initial condition
% initial and boundary condition for the potential
w=zeros(N,1); % w is the potential unit V
for i=1:N1
    w(i)=log(-ni(i)/Tol(i))/si+0.001*(i-1); % this is the calculated potential in the acceptor(p) region.
end
for i=N2+1:N
    w(i)=log(Tol(i)/ni(i))/si+0.001*(i-N);% this is the calculated potential in the donor(n) region.
end
% boundary condition for p, n, and w
VT=NP.VT;
w=w/VT;
Js=0;
wbf=w(1);

wbn=log(Tol(N)/ni(N))/si;

wbn=wbn/VT;% normolized potential
pm=p(2:end-1);
nm=n(2:end-1);
%%
% find a initial guess for the problem for large voltage
if initial_state==0
    if Bias < 1
        w(1)=wbf-(1)/VT;
        % boundary condition for p, n, and w
        wb1=w(1);
        
        
        
        for i=2:N-1
            w(i)=((w(N)-w(1))/(N-1)^2)*((i-1)^2)+w(1);
        end
        x0=[pm;nm;w(2:end-1)];% initial guess of the solution
        fd1=fd(mesh1);
        
        ss1 = Semiconductor_Device(NP,Temperature,Length,mesh1,fd1,Tol,un,up,tn,tp,ni,w,Vps,Vns,Ep,Diameter,DiameterJ,gamma,n1,p1,Impact_Ion,Ae,Be,Ah,Bh,Franz); % Create the Device and initial condition.
        DD0=Drift_Diff_N(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss1);
        gama=ss1.gama;% paprameter in the hole mobility
        opts = optimset('Display','iter-detailed','TolFun',1e-8,'MaxIter',100,'Tolx',1e-6,'MaxIter',40000);
        opts = optimset(opts, 'Diagnostics','on','Jacobian','on','DerivativeCheck','off');
        [x,fval,exitflag,output,jacobian]=fsolve(@DD0.Newton_FD,x0,opts);
        pm=x(1:N-2);
        nm=x(N-1:2*N-4);
        wm=x(2*N-3:3*N-6);
        
    else
        for Bias0=1:0.5:Bias
            w(1)=wbf-(Bias0)/VT;
            % boundary condition for p, n, and w
            wb1=w(1);
            
            
            
            for i=2:N-1
                w(i)=((w(N)-w(1))/(N-1)^2)*((i-1)^2)+w(1);
            end
            x0=[pm;nm;w(2:end-1)];% initial guess of the solution
            fd1=fd(mesh1);
            
            ss1 = Semiconductor_Device(NP,Temperature,Length,mesh1,fd1,Tol,un,up,tn,tp,ni,w,Vps,Vns,Ep,Diameter,DiameterJ,gamma,n1,p1,Impact_Ion,Ae,Be,Ah,Bh,Franz); % Create the Device and initial condition.
            DD0=Drift_Diff_N(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss1);
            gama=ss1.gama;% paprameter in the hole mobility
            opts = optimset('Display','iter-detailed','TolFun',1e-8,'MaxIter',100,'Tolx',1e-6,'MaxIter',40000);
            opts = optimset(opts, 'Diagnostics','on','Jacobian','on','DerivativeCheck','off');
            [x,fval,exitflag,output,jacobian]=fsolve(@DD0.Newton_FD,x0,opts);
            pm=x(1:N-2);
            nm=x(N-1:2*N-4);
            wm=x(2*N-3:3*N-6);
        end
    end
    
        
    else
        pm=p_ini(2:end-1)/Ni;
        nm=n_ini(2:end-1)/Ni;
        wm=w_ini(2:end-1)/VT;
        
    end
%%

    
for resis_i=1:1
        w(1)=wbf-(Bias-abs(Js)*R_load)/VT;
        % boundary condition for p, n, and w
        wb1=w(1);
        

%         for i=2:N1
%             w(i)=((w(N)-w(1))/(N-1)^2)*((i-1)^2)+w(1);
%         end
         x0=[pm;nm;wm];
%         if initial_state==0
%             x0=[pm;nm;wm];
%             % initial guess of the solution
%         else
%            
%         end
        %
        % fid=fopen('Results-IV\6\x.txt');
        % x=fscanf(fid,'%f');
        % fclose(fid);
        % p=x(1:N)/Ni;
        % n=x(N+1:2*N)/Ni;
        % w=x(2*N+1:3*N)/VT;
        
        %mesh1 = mesh_N(Lmin,Lmax,Length,NX);
        %mesh1.Updatemesh(Lmin,Lmax,Length);
        fd1=fd(mesh1);
        
        ss1 = Semiconductor_Device(NP,Temperature,Length,mesh1,fd1,Tol,un,up,tn,tp,ni,w,Vps,Vns,Ep,Diameter,DiameterJ,gamma,n1,p1,Impact_Ion,Ae,Be,Ah,Bh,Franz); % Create the Device and initial condition.
        DD0=Drift_Diff_N(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss1);
        
        gama=ss1.gama;% paprameter in the hole mobility
        opts = optimset('Display','iter-detailed','TolFun',1e-8,'MaxIter',100,'Tolx',1e-6,'MaxIter',40000);
        opts = optimset(opts, 'Diagnostics','on','Jacobian','on','DerivativeCheck','off');
        [x0,fval,exitflag,output,jacobian]=fsolve(@DD0.Newton_FD,x0,opts);



    DD1 = Drift_Diff_N_G(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss1,NP,GL);
    DD2 = Drift_Diff_N_G(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss1,NP,GL);
    %DD2=Drift_Diff_NG(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss,NP);

% for i=2:N-1
%     w(i)=((w(N)-w(1))/(N-1)^2)*((i-1)^2)+w(1);
% end


% read x from file
% fid=fopen('\x.txt');  
% x0=fscanf(fid,'%f');    
% fclose(fid); 

%x=[x;GL];
[x,fval,exitflag,output,jacobian]=fsolve(@DD1.Newton_FD,x0,opts);
%[x,fval,exitflag,output,jacobian]=fsolve(@DD2.Newton_FD,x,opts);
% 
pm=x(1:N-2);
nm=x(N-1:2*N-4);
wm=x(2*N-3:3*N-6);
[~,~,~,Jp,Jn]=DD1.Cal_Current(pm,nm,wm,0,GL,0);% first zeros for time, last 0 for the frequency
J0=NP.J0;
NX=NP.NX;
Ni=NP.Ni;
NT=NP.NT;
Js1=trapz(mesh1.Lx_half,(((Jp+Jn)*J0).*DiameterJ(1:end-1).^2)/sum(mesh1.dx)*0.25*pi);

%J1=(sum(J1.*mesh1.dx)/sum(mesh1.dx))*J0*pi*(4.2e-4)^2;
% wb1=wb1+Js*50/NP.VT;
% if capped
%     remda(1:N-1,1)=1e-6/NP.NX;
%     for i=2:N
%         
%         if mesh1.Lx(i)*NP.NX>=950e-7 & mesh1.Lx(i)*NP.NX<1000e-7
%             remda(i-1)=0.7e-7/NP.NX;
%         elseif mesh1.Lx(i)*NP.NX>=1000e-7 & mesh1.Lx(i)*NP.NX<1750e-7
%             remda(i-1)=5e-7/NP.NX;
%         elseif mesh1.Lx(i)*NP.NX>=1750e-7 & mesh1.Lx(i)*NP.NX<1950e-7
%             remda(i-1)=2.5e-7/NP.NX;
%         elseif mesh1.Lx(i)*NP.NX>=1950e-7
%             remda(i-1)=8e-7/NP.NX;
%         end
%     end
%     %DD1 = Drift_Diff_N_C2(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss,NP,GL,BF,remda);
%     DD2 = Drift_Diff_N_C_GN2(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss,NP,BF,remda);
% else
%     %DD1 = Drift_Diff_N_C(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss,NP,GL,BF);
%     DD2 = Drift_Diff_N_C_GN(pb1,pbn,nb1,nbn,wb1,wbn,fd1,mesh1,ss,NP,BF);
% end


if abs(Js1-Js)/abs(Js1)<1e-4
    Js=Js1;
    break
else
    Js=Js1;
end
% subplot(211)
%  %plot([1:ti]*deltat,pevol,'gx')
%   plot(L(2:N),E,'g')
%   hold on;
% % 
% % subplot(212)
% plot(L,log10(p),'r')
% hold on
% plot(L,log10(n),'r')
%%%%%%%%%%%%%%%%%%%  
% plot(L,log10(abs(Tol+p-n)),'r')
end


p=Ni*[pb1;pm;pbn];
n=Ni*[nb1;nm;nbn];
w=VT*[wb1;wm;wbn];
%Doping_ion=Doping_ion*Ni;
%%%%%%%%%%%%%

L=NX*mesh1.Lx;
%%%%%%%%%%%
E=-(fd1.diffmp(w))/NX;
Js
% hold on
% plot(L(2:end),E,'r','linewidth',2);



