% this is the main code to calculate the responsivity as a function of bias
% for fixed optical power.
clear
close all
P1=18.3; % this is the power of the light of input i unit of mW
P0=P1*0.9344; % 0.9344 is the quantum efficience, this is a fitting parameter to fit the experimental results
for main_i=3:10 % this is the loop to calculate the steady state output current for different bias, 
    %here we use parfor for the parallel computing
    deltaV=1; % the bias change , here we did not use it.
    BIAS1=main_i/2; % the bias applied on the PD. reversed bias.
    
    %     G0=15.5526*1.1;%10.5526 21.0526/17 for Campbell Franz unit power light mW.conresponse to the light generation of 0.0012 % 1.2 % 0.12
    %     G00=G0;
    % the following 4 parameters are the impact ionization fitting
    % parameters
    % Ae and Be are the electron impact ionization, in the InGaAs, the holes impact ionization is small compared to the electrons.
    % so Ah is 0 here.
    Ae=1.8e7;
    Be=1.87e6;
    Ah=0;
    Bh=2.26e6;
    Impact_Ion=1; % impact ionization turn on/off
    BF=0;
    DeltaEA=0.025; % the  acceptor doping density level
    DeltaED=0.005; % the  donor doping density level
    % the main function to calculate the steady state of the pd under the
    % illumination of optical power.\
    Ini=0; % the inital condition is 0  
    [p,n,w,Js]=SteadyPDsolver(BIAS1,BF,P0,0,Ae,Be,Ah,Bh,Ini,0,0,0,0,deltaV,DeltaEA,DeltaED);
    Ini=1; % We use the previous results as the initial condition, so ,we set Ini=1;
    [p,n,w,Js]=SteadyPDsolver(BIAS1,BF,P0,Impact_Ion,Ae,Be,Ah,Bh,Ini,p,n,w,Js,deltaV,DeltaEA,DeltaED);
    JC(main_i)=Js; % the output of SteadyPDsover is hole density p, electron density n, potential w, and current J.
end
Responsivity % the experimental responsivity data
hold on
plot(1.5:0.5:5,-JC(3:10)./(0.001*P1),'linewidth',2) % plot the simulation responsivity data
% set up the figure
legend('Experiment','Simulation')
xl = xlabel('Bias (V)');
yl = ylabel('Responsivity (A/W)');
set(gca, 'FontSize',14,'FontName','Times New Roman')
set(xl, 'FontSize',14,'FontName','Times New Roman')
set(yl, 'FontSize',14,'FontName','Times New Roman')
set(gcf, 'PaperPosition',[0 0 4 3],'PaperSize',[4 3])
set(gca, 'XMinorGrid', 'off')
box on

