classdef Semiconductor_Device < handle
    %SEMICONDUCTOR_DEVICE Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
         % total nodes
        Length% the device length unit cm
        mesh % the class mesh
        %dx % grid profile array(N-1), the distance between Nodes, unit cm
        fd % Finite difference object
        
        Doping_Profile% doping profile of the device ,
        %it should be a array. unit cm-3, acceptor is minus, donor is positive
        
        Electron_Mobility % the electron mobility unit,cm^2/(V*s)
        Hole_Mobility % the hole mobility unit,cm^2/(V*s)
        
        
        Lifetime_n % recombination lifetime for electron
        
        Lifetime_p % recombination lifetime for hole
     
        Temperature % Temperature unit K
        T_Vol % electron voltage. unit V, T_Vol=KB*T/q; q is the charge
        
        Hole_Density% hole density profile array of N, unit cm-3
        Elec_Density% electron density profile array of N unit cm-3
        Potential% Potential Voltage unitV
        Elec_Field % Electrical Field unit V/cm
        Ep% calculated saturted electrical field parameter
        Intrinsic_Density % intrinsic density
        P_Vsaturate
        N_Vsaturate
        Diameter% Diameter of the Beam
        DiameterJ % diameter to calculate the current
        NP % normlized parameter class
        gama % the parameter in the hole velocity, to control the increase of the velocity as a function of E
        n1 % the ionization density
        p1 % the ionization denstiy
	    Impact_Ion % sign for the impact ionization model
       Ae % impact ionization parameter unit cm-1
       Be % impact ionization parameter unit V/cm 
       Ah % impact ionization parameter unit cm-1
       Bh % impact ionization parameter unit V/cm
       Franz % franz-keldysh effect.
      Mater % material properities
      alpha_p % absoprtion in p as a percent in the i region, 0.05
      alpha_n% absorption in n as a percent in the i region, 0.05
      In % impact ionization
      Ip % impact ionization
	alpha0

    end
    properties (Constant)
        KB=1.38e-23;% Boltzmann constant,unit,J/K
        q=1.6e-19;
        Ebslon=1.21e-12; % unit F/cm; %pertermis
        %gama=1;
        
       
        %
              
    end
    methods
       

        
        function obj=Semiconductor_Device(NP,Temperature,Length,mesh,fd,Doping_Profile,Electron_Mobility,Hole_Mobility,Lifetime_n,Lifetime_p,...
                Intrinsic_Density,Potential,P_Vsaturate,N_Vsaturate,Ep,Di,DiameterJ,gamma,n1,p1,Impact_Ion,Ae,Be,Ah,Bh,Franz,Mater,alpha_p,alpha_n,In,Ip,alpha0)
            % the Doping_Profile should be the array. And the length of
            % Doping Profile should be equal to the "Part"
            obj.NP=NP;
            N=length(mesh.dx)+1;
            obj.Temperature=Temperature;
            obj.Length=Length;
            obj.mesh=mesh;
            obj.fd=fd;
            obj.Doping_Profile=Doping_Profile;
            obj.T_Vol=obj.KB*Temperature/obj.q;
            obj.Electron_Mobility=Electron_Mobility;
            obj.Hole_Mobility=Hole_Mobility; 
            obj.Lifetime_n=Lifetime_n;
            obj.Lifetime_p=Lifetime_p;
            obj.Intrinsic_Density=Intrinsic_Density;
            obj.Hole_Density=abs(min(Doping_Profile,zeros(N,1)));% initial the Hole density to the acceptor doping
            obj.Elec_Density=abs(max(Doping_Profile,zeros(N,1)));% initial the Hole density to the donor doping
            obj.Potential=Potential;
            obj.Elec_Field=-fd.diffmp(obj.Potential);
            obj.P_Vsaturate=P_Vsaturate;
            obj.N_Vsaturate=N_Vsaturate;
            obj.Ep=Ep;
            obj.Diameter=Di;
            obj.DiameterJ=DiameterJ;
            obj.gama=gamma;
            obj.n1=n1;
            obj.p1=p1;
            obj.Impact_Ion=Impact_Ion;
            obj.Ae = Ae;
            obj.Be = Be;
            obj.Ah=Ah;
            obj.Bh=Bh;
            obj.Franz=Franz;
            obj.Mater=Mater;
            obj.alpha_p=alpha_p;
            obj.alpha_n=alpha_n;
            obj.In = In;
            obj.Ip = Ip;
	     obj.alpha0 = alpha0;

        end
        

        function Update(self,p,n,phi)
            % update the carrier density and potential
            self.Hole_Density=p;
            self.Elec_Density=n;
            self.Potential=phi;
        end
        function Plotn(self,x)
            % x is the variable you want to plot
            % the normal coordinalator
            plot(self.mesh.Lx,x);
        end
        function Plotl(self,x)
            % x is the variable you want to plot
            % plot log function
            plot(self.mesh.Lx,log10(x));
        end
        function [p,n,phi]=Get(self)
            % to get the carrier and potential function 
            p=self.Hole_Density;
            n=self.Elec_Density;
            phi=self.Potential;
        end

            

    end
    
end

